#!/bin/bash

CACHE_DIR=/var/spool/up2date
ROOT_DIR=/var/www
LOG_FILE=/var/log/vzrhnproxy
RHN_CONF_DIR=/etc/sysconfig/rhn
DOWNLOAD_URL="/rhn/download"
MIRRORS_URL="/rhn/download/mirrors"
BROWSE_URL="/rhn/browse"

FUNCTIONS="/usr/share/vzrhnproxy/functions"
[ -f "$FUNCTIONS" ] || exit 3
. $FUNCTIONS

COPY_STD="/usr/share/vzrhnproxy/copy_std"
STAT="/usr/bin/stat"
UP2DATE=/usr/sbin/up2date



function weblog()
{
	echo -e "`date`  $*" >> $LOG_FILE
}

function gen_error()
{
	
	weblog "ERROR: $*"

        header_html

        echo "<html>"
        echo "ERROR"
        echo "<hr>"
        echo $URL
        echo "<hr>"
        echo $*
        echo "</html>"

	exit 0
}

function get_size()
{
        [ $# -eq 1 ] || abort "usage: $FUNCTION file"
        local file=$1 rc
        $STAT $file | grep "Size" | sed "s#.*Size: \([0-9]*\).*#\1#g"
        rc=$?
        [ $rc -eq 0 ] || gen_error "can't stat '$file'"
}


function header_mirrorlist()
{

	local file=$1 size=$2
	[ "$size" ] || size=`get_size $file`

	echo "Content-Length: $size"
	echo "Connection: close"
	echo "Content-Type: text/plain; charset=UTF-8"
	echo
}

function header_html()
{
	echo "Content-Type: text/html; charset=UTF-8"
	echo
}

function header_data()
{
	local file=$1 size=$2 filename
	[ "$size" ] || size=`get_size $file`
	filename=`echo $file | sed "s#.*/\(.*\)#\1#g"`

	echo "Accept-Ranges: bytes"
	echo "Content-Length: $size"
	echo "Connection: close"
	echo "Content-Type: application/octet-stream; name=$filename"
	echo "Content-Disposition: attachment; filename=$filename"
	echo 
}

function search_configs_by_ip()
{
	local ip=$1 configs file found=0

	configs=`list_configs`	
	for file in $configs ; do
		grep "^IPS" $file | grep -q "${ip}"
		if [ $? -eq 0 ] ; then
			found=1 
			echo ${file}
		fi
	done
	[ ${found} -eq 1 ] && return 0 || return 1

}

function correct_arch()
{
	local arch=$1

	case $arch in
		i386|i486|i586|i686|x86)
			echo "i386"
			return 0
		;;
	esac
	echo $arch
}

function check_config()
{
	local config=$1 arch=$2 os_ver=$3 wo_arch check
#	config=${config%/*}
#	config=${config##*/}

	grep -q "^ARCH=$arch" $config
	[ $? -eq 0 ] || return 1

	if [ $os_ver == "4EL" ] ; then
		grep -E -q "^OS_VERSION=4AS|^OS_VERSION=4ES" $config
		[ $? -eq 0 ] || return 1
	fi

	return 0
}

#parse url and return config path 
function search_config()
{
	local repo_path url=$1 ip=$2 configs t_url os_ver arch i

	configs=`search_configs_by_ip $ip`
	[ $? -eq 0 ] || return 1
	
	t_url=${url#${DOWNLOAD_URL}"/"}
	
	[ "${t_url}" = "${url}" ] && return 1

	os_ver=${t_url%%/*}
	t_url=${t_url#${os_ver}"/"}

	arch=${t_url%%/*}

	arch=`correct_arch $arch`

	for i in $configs ; do
		check_config "$i" "$arch" "$os_ver"
		if [ $? -eq 0 ] ; then
			echo $i
			return 0
		fi
	done
	return 1
}




function gen_link()
{

        local file=$1 name gname browse=$2
        name=${file##*/}
        gname=$name
        echo $name | grep -q ".*\.hdr"
        if [ $? -eq 0 ] ; then
                gname=`echo $name | sed "s#\(.*\)\.hdr#\1.rpm#g"`
                name=$gname
        fi

        browse=${browse%$name}
        browse=${browse%"/"}
	browse=${browse#"/"}
        if [ "$browse"  ] ; then
                echo "<a href=$BROWSE_URL/${browse}/$name > $name </a>"
        else
                echo "<a href=$BROWSE_URL/$name > $name </a>"
        fi
}

function download()
{
	local url=$1 ip=$2 file name full_name range size session_conf path line spool_dir rc
	name=${url##*/}
	# search according repository and config file
	session_conf=`search_config $url $ip`
	[ $? -eq 0 ] || gen_error "config for system not found, may be system is unregistered"

	line=`read_session_conf $session_conf`
	rc=$?
	[ $rc -eq 0 ] || gen_error "during read config '$session_conf'"
	set $line
	spool_dir=$3
	[ "${spool_dir}" ] || gen_error "SPOOL_DIR is not defined"

#	file=${file#/sc}
	full_name="$spool_dir/$name"

	if [ ! -f $full_name ] ; then
		#download from redhat via up2date according to session config
		[ -f $session_conf ] || gen_error "can't find session config"
		[ "$session_conf" ] || gen_error "spool directory is not defined"
		sudo $UP2DATE --vzconfig $session_conf --get $name 2>&1 >> $LOG_FILE
		rc=$?
		weblog "download $name  rc=$rc"
		[ $rc -eq 0 ] || gen_error "can't download $name"
	fi

	if [ "$HTTP_RANGE" ] ; then
		 echo "Status: 206 Partial Content"
		 #echo "Content-Range: $HTTP_RANGE"
		 sta=`echo $HTTP_RANGE | sed "s#.*=\([0-9]*\).*#\1#g"`
		 end=`echo $HTTP_RANGE | sed "s#.*-\([0-9]*\).*#\1#g"`
 		 size=`get_size $full_name`
		 echo "Content-Range: bytes $sta-$end/$size"
		 range=" -s $sta -e $end "
		 size=$((end-$sta+1))
	else
		 echo "Status: 200 OK"
	fi


	
	[ "$size" ] && 	header_data $full_name $size || \
		header_data $full_name

	#read package and print to stdout
	$COPY_STD $range $full_name
}

function download_xml()
{
	local url=$1 ip=$2 name full_name spool_dir session_conf line rc

	name=${url##*/}
	# search according repository and config file
	session_conf=`search_config $url $ip`
	rc=$?
	[ $rc -eq 0 ] || gen_error "accordind to '$url', '$ip' config is not found"

	line=`read_session_conf $session_conf`
	rc=$?
	[ $rc -eq 0 ] || gen_error "during read config '$session_conf'"
	set $line
	spool_dir=$3
	[ "${spool_dir}" ] || gen_error "SPOOL_DIR is not defined"

	full_name="$spool_dir/repodata/$name"

	if [ ! -f $full_name ] ; then
		gen_error "file not found: $full_name"
	fi

	header_data $full_name 
	$COPY_STD $full_name
}

function download_mirrorlist()
{
	local file=$1 name full_name
	full_name=$ROOT_DIR/$file

	if [ ! -f $full_name ] ; then
		gen_error "file '$full_name' is not found"
	fi

	header_mirrorlist $full_name
	$COPY_STD $full_name 
}


function show_list()
{
	local files link browse

	browse=${URL#$BROWSE_URL"/"}
	header_html

	echo "<html>"
	echo "request: $URL"
	echo "<hr>"
	files=`ls ${CACHE_DIR}/${browse}`

	for i in $files ; do
		link=`gen_link $i $browse`
		echo "$link"
		echo "<br>"
	done

	echo "</html>"
}



weblog "request: $REDIRECT_URL from $REMOTE_ADDR"

URL="$REQUEST_URI"
if [  "$REDIRECT_URL" ] ; then
	URL="$REDIRECT_URL"
fi


if [ "${URL#$BROWSE_URL}" != "$URL"  ] ; then
        show_list $URL
elif [ "${URL%.rpm}" != "${URL}" ] ; then
	download "$URL" "$REMOTE_ADDR"
elif [ "${URL%.xml}" != "${URL}"  ] ; then
	download_xml "$URL" "$REMOTE_ADDR"
elif [ "${URL%.xml.gz}" != "${URL}"  ] ; then
	download_xml "$URL" "$REMOTE_ADDR"
elif [ "${URL%/*}" == "$MIRRORS_URL"  ] ; then
	download_mirrorlist $URL	
fi

exit $?
